
/*  @(#)xutil.c 1.5 95/02/06
 *
 *  Various routines associated with the input popup.
 *
 *  Copyright (c) Rich Burridge.    All rights reserved.
 *
 *  Permission is given to distribute these sources, as long as the
 *  copyright messages are not removed, and no monies are exchanged.
 *
 *  No responsibility is taken for any errors or inaccuracies inherent
 *  either to the comments or the code of this program, but if
 *  reported to me then an attempt will be made to fix them.
 */

#include "mptool.h"
#include <Xm/List.h>
#include <Xm/Text.h>
#include <Xm/MessageB.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>

#ifndef NO_DND
#include <Xm/DragDrop.h>

static void drop_proc        (Widget, XtPointer, XtPointer) ;
static void transfer_proc    (Widget, XtPointer, Atom *, Atom *,
                                      XtPointer, unsigned long *, int) ;
#endif /*!NO_DND*/

Widget npopup ;              /* Popup information dialog. */


void
add_delete_callback(Widget widget)
{
  Atom WM_DELETE_WINDOW ;

  WM_DELETE_WINDOW = XmInternAtom(XtDisplay(widget),
                                  "WM_DELETE_WINDOW", False) ;
  XmAddWMProtocolCallback(widget, WM_DELETE_WINDOW,
                          (XtCallbackProc) dismiss_popup, NULL) ;
}


void
add_list_item(Widget list, int index, char *label)
{
  XmString str ;
 
  str = XmStringCreateSimple(label) ;
  XmListAddItemUnselected(list, str, index) ;
  XmStringFree(str) ;
}


/*  Minimal routine to create a pixmap for the two color mptool icons. This
 *  routine assumes that the data files are in the correct format.
 */
 
Pixmap
create_mp_pixmap(char *buf)
{
  char c_ch[MAXICONCOLORS], c_str[MAXICONCOLORS][MAXLINE] ;
  char *p ;
  Colormap cmap ;
  GC gc ;
  int height, i, j, ncolors, nchars, width ;
  Pixmap pixmap = 0 ;
  unsigned char *data, *dp ;
  unsigned char lkup_tbl[256] ;
  XColor ccol ;
  XImage *image ;

  p = buf ;

/* Extract width, height, number of colors and number of characters/pixel. */

  SSCANF(p, "%d %d %d %d", &width, &height, &ncolors, &nchars) ;
  while (*p != '\n') p++ ;
  p++ ;

  for (i = 0; i < ncolors; i++)
    {
      c_ch[i] = *p ;
      SSCANF(p+4, "%s", c_str[i]) ;
      while (*p != '\n') p++ ;
      p++ ;
    }

  for (i = 0; i < 256; i++) lkup_tbl[i] = 0 ;
  cmap = DefaultColormap(dpy, screen) ;
  for (i = 0; i < ncolors; i++)
    if (XParseColor(dpy, cmap, c_str[i], &ccol))
      if (XAllocColor(dpy, cmap, &ccol) == True)
        lkup_tbl[(unsigned int) c_ch[i]] = (unsigned char) ccol.pixel ;

  data = (unsigned char *) malloc((unsigned)(width * height)) ;
  if (data == NULL) return(pixmap) ;

  dp = data ;
  for (i = 0; i < height; i++)
    {
      for (j = 0; j < width; j++)
        *dp++ = lkup_tbl[(unsigned int) *p++] ;
      while (*p != '\n') p++ ;
      p++ ;
    }
  gc = DefaultGC(dpy, screen) ;
  image = XCreateImage(dpy, DefaultVisual(dpy, screen),
                       depth, ZPixmap, 0, (char *) data,
                       width, height, 8, 64) ;
  pixmap = XCreatePixmap(dpy, root, width, height, depth) ;
  XPutImage(dpy, pixmap, gc, image, 0, 0, 0, 0, width, height) ;
  return(pixmap) ;
}


void
delete_list_item(Widget list, int index)
{
  XmListDeletePos(list, index) ;
}


void
dismiss_popup(Widget widget, XtPointer p, XmAnyCallbackStruct *s)
{
  XtUnmapWidget(widget) ;
  XtUnmanageChild(widget) ;
}


#ifndef NO_DND
static void
drop_proc(Widget widget, XtPointer p, XtPointer call_data)
{
  XmDropProcCallback     DropData ;
  XmDropTransferEntryRec transferEntries[2] ;
  XmDropTransferEntry    transferList ;
  Arg                    args[10] ;
  int                    n = 0 ;

  DropData = (XmDropProcCallback) call_data ;

/* If the action is not Drop or the operation is not Copy, cancel the drop. */

  if ((DropData->dropAction != XmDROP) ||
      (DropData->operation != XmDROP_COPY))
    {
      XtSetArg(args[n], XmNtransferStatus,   XmTRANSFER_FAILURE) ; n++ ;
    }
  else
    { 

/* The drop can continue. Establish the transfer list and start transfer. */

      transferEntries[0].target      = COMPOUND_TEXT ;
      transferEntries[0].client_data = (XtPointer) widget ;
      transferList = transferEntries ;
      XtSetArg(args[n], XmNdropTransfers,    transferList) ;         n++ ;
      XtSetArg(args[n], XmNnumDropTransfers, 1) ;                    n++ ;
      XtSetArg(args[n], XmNtransferProc,     transfer_proc) ; n++ ;
    }

/* Start the transfer or cancel. */

  XmDropTransferStart(DropData->dragContext, args, n) ;
}
#endif /*!NO_DND*/


void
force_popup_on_screen(Widget popup, int *popup_x_p, int *popup_y_p)
{
  Dimension popup_width, popup_height ;
  Position left, top ;
  int popup_x, popup_y ;
  int n, screen_width, screen_height ;

  popup_x = *popup_x_p ;
  popup_y = *popup_y_p ;

/* Get the screen size. */

  get_screen_size(&screen_width, &screen_height) ;

  XtVaGetValues(popup,
                XmNwidth,  &popup_width,
                XmNheight, &popup_height,
                0) ;

/* Make sure frame does not go off side of screen. */

  n = popup_x + (int) popup_width ;
  if (n > screen_width) popup_x -= (n - screen_width) ;
  else if (popup_x < 0) popup_x = 0 ;

/* Make sure frame doen't go off top or bottom. */
     
  n = popup_y + (int) popup_height ;
  if (n > screen_height) popup_y -= n - screen_height ;
  else if (popup_y < 0) popup_y = 0 ;
    
/* Set location and return. */

  left = (Position) popup_x ;
  top  = (Position) popup_y ;
  XtVaSetValues(popup,
                XmNx, left,
                XmNy, top,
                0) ;

  *popup_x_p = popup_x ;
  *popup_y_p = popup_y ;
}


int
get_bool_value(enum frame_type ftype, int index)
{
  Boolean value ;

  XtVaGetValues(get_widget(ftype, index), XmNset, &value, 0) ;
  return(value) ;
}


int
get_int_value(enum frame_type ftype, int index)
{
  return(atoi(XmTextGetString(get_widget(ftype, index)))) ;
}


void
get_screen_size(int *width_p, int *height_p)
{
  *width_p  = DisplayWidth(dpy,  screen) ;
  *height_p = DisplayHeight(dpy, screen) ;
}


int
get_scroll_nrows(Widget widget)
{
  int n ;

  XtVaGetValues(widget, XmNitemCount, &n, 0) ;
  return(n) ;
}


char *
get_str_value(enum frame_type ftype, int index)
{
  return(XmTextGetString(get_widget(ftype, index))) ;
}


#ifndef NO_DND
void
init_dnd(Widget widget)
{
  Arg args[5] ;
  Atom import_list[1] ;
  int n = 0 ;

  import_list[0] = COMPOUND_TEXT ;
  XtSetArg(args[n], XmNimportTargets,      import_list) ;        n++ ;
  XtSetArg(args[n], XmNnumImportTargets,   1) ;                  n++ ;
  XtSetArg(args[n], XmNdropSiteOperations, XmDROP_COPY) ;        n++ ;
  XtSetArg(args[n], XmNdropProc,           drop_proc) ;          n++ ;
  XmDropSiteRegister(widget, args, n) ;
}
#endif /*!NO_DND*/


int
is_window_showing(Widget widget)
{
  return(XtIsManaged(widget)) ;
}


int
item_exists(Widget widget, char *str)
{
  XmString xstr = XmStringCreateSimple(str) ;
  int pos ;

  pos = XmListItemPos(widget, xstr) ;
  XmStringFree(xstr) ;
  return(pos) ;
}


void
notice(Widget widget, char *message)
{
  XmString contstr = XmStringCreateSimple("Continue") ;
  XmString messstr ;
 
  npopup = (Widget) XmCreateWarningDialog(widget, "notice", NULL, 0) ;
  XtVaSetValues(npopup,
                XmNautoUnmanage,      FALSE,
                XmNcancelLabelString, contstr,
                XmNdialogStyle,       XmDIALOG_FULL_APPLICATION_MODAL,
                0) ;
  XtVaSetValues(XtParent(npopup), XmNtitle, "Notice", 0) ;
  XtUnmanageChild((Widget) XmMessageBoxGetChild(npopup, XmDIALOG_OK_BUTTON)) ;
  XtUnmanageChild((Widget) XmMessageBoxGetChild(npopup,
                                                XmDIALOG_HELP_BUTTON)) ;
 
  messstr = XmStringCreateLtoR(message, XmSTRING_DEFAULT_CHARSET) ;
  XtVaSetValues(npopup, XmNmessageString, messstr, 0) ;
  XtManageChild(npopup) ;
  XtPopup(XtParent(npopup), XtGrabNone) ;
 
  XmStringFree(contstr) ;
  XmStringFree(messstr) ;
}


void
position_popup(Widget base, Widget popup)
{
  int bw, bx, by, px, py ;
  int screen_width, screen_height ;
  Position base_x, base_y, popup_x, popup_y ;
  Dimension base_width, base_height, popup_width, popup_height ;

  XtVaGetValues(base,
                XmNx,      &base_x,
                XmNy,      &base_y,
                XmNwidth,  &base_width,
                XmNheight, &base_height,
                0) ;
  bx = (int) base_x ;
  by = (int) base_y ;
  bw = (int) base_width ;

  XtVaGetValues(popup,
                XmNx,      &popup_x,
                XmNy,      &popup_y,
                XmNwidth,  &popup_width,
                XmNheight, &popup_height,
                0) ;

  px = (int) popup_x ;
  py = (int) popup_y ;

  get_screen_size(&screen_width, &screen_height) ;
  px = bx + bw + 5 ;
  py = by - TITLE_LINE_HEIGHT ;
  force_popup_on_screen(popup, &px, &py) ;
}


void
set_active(enum frame_type ftype, int index, int state)
{
  XtSetSensitive(get_widget(ftype, index), state) ;
}


void
set_bool_value(enum frame_type ftype, int index, int n)
{
  Boolean value = n ;

  XtVaSetValues(get_widget(ftype, index), XmNset, value, 0) ;
}


void
set_excl_value(enum frame_type ftype, int index)
{
  Cardinal count ;
  int i ;
  Widget w      = get_widget(ftype, index) ;
  Widget parent = XtParent(w) ;
  WidgetList kids ;

  XtVaGetValues(parent,
                XmNnumChildren, &count,
                XmNchildren,    &kids,
                0) ;
  for (i = 0; i < count; i++)                   /* Clear all values first. */
    XtVaSetValues(kids[i], XmNset, False, 0) ;
  XtVaSetValues(w, XmNset, True, 0) ;
}


void
set_int_value(enum frame_type ftype, int index, int value)
{
  char str[MAXLINE] ;

  SPRINTF(str, "%d", value) ;
  XmTextSetString(get_widget(ftype, index), str) ;
}


void
set_list_position(Widget widget, int pos)
{
  XmListSetBottomPos(widget, pos) ;
}


void
set_str_value(enum frame_type ftype, int index, char *str)
{
  char dummy[1] ;

  if (!str)
    {
      str = dummy ;
      *dummy = '\0' ;
    }
  XmTextSetString(get_widget(ftype, index), str) ;
}


void
show_popup(Widget widget)
{
  if (!is_window_showing(widget))
    position_popup(warr[WI_MAIN_FRAME], widget) ;
  XtMapWidget(widget) ;
  XtManageChild(widget) ;
  XRaiseWindow(dpy, XtWindow(widget)) ;
}
 
 
#ifndef NO_DND
static void
transfer_proc(Widget widget, XtPointer p, Atom *a, Atom *type,
                             XtPointer value, unsigned long *l, int i)
{
  char err[MAXLINE], filename[MAXLINE] ;
  FILE *fp ;
 
/* Convert information from the drag initiator into a compund string. */
 
  if (*type == COMPOUND_TEXT)
    {
      STRCPY(filename, "/tmp/.MptoolXXXXXX") ;
      MKTEMP(filename) ;
      if ((fp = fopen(filename, "w")) != NULL)
        {
          FPUTS((char *) value, fp) ;
          FCLOSE(fp) ;
          do_print_cmd(filename) ;
        }
      else
        { 
          SPRINTF(err, mptool_strs[(int) S_DRAGDROP], filename) ;
          notice(warr[WI_MAIN_FRAME], err) ;
        }
    }    
}
#endif /*!NO_DND*/


void
Xutil(void)
{
  npopup = NULL ;       /* No notice popup window initially. */
}
